@@ -30,12 +30,13 @@ from tensorflow.keras.callbacks import TensorBoard
 import json
 from scivae import Loss
 from sciutil import SciException, SciUtil
+import random
+import string
 
 
 class VAEException(SciException):
     def __init__(self, message=''):
         Exception.__init__(self, message)
-c = "#69d"
 
 
 class VAE(object):
@@ -51,11 +52,14 @@ class VAE(object):
         https://github.com/tensorflow/tensorflow/issues/41053 for saving
     """
 
-    def __init__(self, input_data_np: np.array, output_data_np: np.array, labels: list, config, vae_label='',
+    def __init__(self, input_data_np: np.array, output_data_np: np.array, labels: list, config, vae_label=None,
                  sciutil=None, config_as_str=False, empty=False):
         self.u = sciutil if sciutil is not None else SciUtil()
         # Setting the other internal variables
         # Check if the config is a string object
+        if vae_label is None:
+            rand_label = ''.join(random.choices(string.ascii_uppercase + string.digits, k=8))
+            vae_label = f'VAE_{rand_label}'
         if config_as_str:
             with open(config, "r") as fp:
                 config = json.load(fp)  # load from the FP
@@ -101,7 +105,7 @@ class VAE(object):
                 return
         self.sample_method = config.get('sample_method') if config.get('sample_method') else "normal"  # Or negbinom
         self.seed = config.get('seed') if config.get('seed') else 17
-        scale_data = config.get('scale_data') if config.get('scale_data') else True
+        scale_data = config.get('scale_data') if config.get('scale_data') else False
         self.batch_norm = True if config.get('batch_norm') else False
         if scale_data and self.input_data_np is not None:
             self.scale_data()
@@ -314,8 +318,8 @@ class VAE(object):
             optimizer.from_config(optimizer_config)
         self.vae.compile(optimizer=optimizer)
 
-
-    def encode(self, method='default', epochs=50, batch_size=50, train_percent=85.0, logging_dir=None, logfile=None):
+    def encode(self, method='default', epochs=50, batch_size=50, train_percent=85.0, logging_dir=None, logfile=None,
+               early_stop=False):
         """
         Encodes the data based on a given method.
 
@@ -341,16 +345,32 @@ class VAE(object):
             msg = self.u.msg.msg_arg_err("encode", "method", method, ['default', 'optimise'])
             self.u.err_p([msg])
             raise VAEException(msg)
-
-        csv_logger = CSVLogger(logfile, append=True, separator=',')
-        # Otherwise fit the VAE with the training and test data
-        self.vae.fit(self.training_input_np,
-                     epochs=epochs,
-                     batch_size=batch_size,
-                     shuffle=True,
-                     validation_data=(self.test_input_np, None),
-                     callbacks=[TensorBoard(log_dir=logging_dir), csv_logger]
-        )
+        # if logging_dir:
+        #     csv_logger = CSVLogger(logfile, append=True, separator=',')
+        #     self.vae.fit(self.training_input_np,
+        #                  epochs=epochs,
+        #                  batch_size=batch_size,
+        #                  shuffle=True,
+        #                  validation_data=(self.test_input_np, None),
+        #                  callbacks=[TensorBoard(log_dir=logging_dir), csv_logger]
+        #     )
+        if early_stop:
+            csv_logger = CSVLogger(logfile, append=True, separator=',')
+            callback = tf.keras.callbacks.EarlyStopping(monitor='loss', patience=3)
+            self.vae.fit(self.training_input_np,
+                            epochs=epochs,
+                            batch_size=batch_size,
+                            shuffle=True,
+                            validation_data=(self.test_input_np, None),
+                         callbacks=[csv_logger, callback]
+                        )
+        else:
+            self.vae.fit(self.training_input_np,
+                         epochs=epochs,
+                         batch_size=batch_size,
+                         shuffle=True,
+                         validation_data=(self.test_input_np, None)
+                         )
 
     def _encode_default(self):
         """
